--=======================================================================================================
-- BALESEE SCRIPT
--
-- Purpose:		Allows bales and pallets to show up on the PDA map as hotspots.
-- Author:		Mmtrx		
-- Changelog:
--  v1.0		03.01.2019	original FS17 version by akuenzi (akuenzi@gmail.com)
--	v1.1.0		28.08.2019	updates for FS19, added user interface  
--  v1.1.1		17.09.2019  added pallette support.
--  v1.1.2		08.10.2019	save statistics / add legend in debug mode 
--=======================================================================================================

BaleSee = {};
BaleSee.version 	= "1.1.2" 		-- in this version the mod creates hotspots for bales and pallets.
BaleSee.bHotspots 	= 	{}			-- to keep track of all generated b-ale and p-allet hotspots
BaleSee.pHotspots	= 	{}			-- each entry is a tuple {hotspot, color, image}

---------------- constants -----------------------------------------------------------------------
BaleSee.debug 		= false
BaleSee.modDir 		= g_currentModDirectory
BaleSee.visible 	=	{[false] = 	"invisible";[true]  =	"visible"}
BaleSee.isIcon 		=	{[false] = 	"icons";	[true]  =	"spots"}
BaleSee.isRound		=	{[false] = 	"square";	[true]  =	"round"}
BaleSee.icons 		=	{
		squareStraw	= "HotspotIcons/StrawSquareBale.dds",
		squareHay	= "HotspotIcons/HaySquareBale.dds",
		squareGrass	= "HotspotIcons/GrassSquareBale.dds",
		squareSilage= "HotspotIcons/SilageSquareBale.dds",
		roundStraw	= "HotspotIcons/StrawRoundBale.dds",
		roundHay	= "HotspotIcons/HayRoundBale.dds",
		roundGrass	= "HotspotIcons/GrassRoundBale.dds",
		roundSilage	= "HotspotIcons/SilageRoundBale.dds",
		woolPallet	= "HotspotIcons/WoolPallet.dds",
		otherPallet	= "HotspotIcons/Pallet.dds",
		-- Transparent image used as background to show hotspot icons 
		-- without the usual default (black) background:
		bgImage		= "HotspotIcons/BGOverlay.dds",	
					}
-- ---------------Helper functions------------------------
source(BaleSee.modDir.."scripts/helper.lua")
-- ---------------Manage Hotspots for bales and pallets---
source(BaleSee.modDir.."scripts/hotspots.lua")
------------------User interface functions ---------------
source(BaleSee.modDir.."scripts/userint.lua")

----------------------- initilization on load Map ----------------------------------------------
function BaleSee:loadMap(name)
	print("--- loading BaleSee V".. BaleSee.version .. " (c) by Mmtrx ---")
	-- SaveSettings
	FSBaseMission.saveSavegame 	= Utils.appendedFunction(FSBaseMission.saveSavegame, BaleSee.saveSettings);		
		
	-- Append functions for bales.
	Bale.delete = 	Utils.appendedFunction(Bale.delete, BaleSee.delete);
	Bale.setNodeId= Utils.appendedFunction(Bale.setNodeId, BaleSee.setNodeId);
	
    -- needed for action event for player
    Player.registerActionEvents = Utils.appendedFunction(Player.registerActionEvents, BaleSee.registerActionEventsPlayer);
	Player.removeActionEvents = Utils.appendedFunction(Player.removeActionEvents, BaleSee.removeActionEventsPlayer);		
		
	-- register the functions to be triggered for pallets (is a vehicle type :)
	local pType =  g_vehicleTypeManager:getVehicleTypeByName("pallet")
	SpecializationUtil.registerEventListener(pType, "onLoadFinished",  	BaleSee);
	SpecializationUtil.registerEventListener(pType, "onDelete", 		BaleSee);	
	SpecializationUtil.registerEventListener(pType, "onChangedFillType",BaleSee);	
	
	-- initialize constants
	BaleSee.ACTIONS =	{ 
		bs_Bale		= BaleSee.actionbs_Bale,
		bs_Color	= BaleSee.actionbs_Color,
		bs_Help		= BaleSee.actionbs_Help,
					}
	BaleSee.colors		=	{  -- set this here, because FillType.x is not yet filled, outside of loadMap()
	[FillType.STRAW] =		{0.6, 	0.3, 	0, 		1},	-- Orange
	[FillType.DRYGRASS_WINDROW] ={0.4, 	1, 		0.4, 	1},	-- Light Green
	[FillType.GRASS_WINDROW]={0.1, 	0.2, 	0, 		1},		-- Dark Green
	[FillType.SILAGE] =		{0.95, 	0.35, 	0.35,	1},		-- pink
	-- pallet big bags:
	[FillType.UNKNOWN] =	{1, 	0, 		1, 		1},		-- Magenta
	[FillType.PIGFOOD] =	{0.27, 	0.085,	0.085, 	1},		-- dark pink
	[FillType.SEEDS] =		{0.1, 	0.04, 	0.04, 	1},		-- dark brown
	[FillType.FERTILIZER] = {0.55, 	0.25, 	0.25,	1},		-- medium pink
	[FillType.LIME] =		{1, 	0.6, 	0.6, 	1},		-- light pink
	[FillType.WHEAT] =		{0.55, 	0.5, 	0.26,	1},		-- light bronze
	[FillType.OAT] =		{0.34, 	0.29, 	0.15,	1},		-- medium bronze			
	-- pallet tanks:
	[FillType.LIQUIDFERTILIZER] = {0.79, 	0.6, 	0.3, 1},-- light Orange
	[FillType.HERBICIDE] =	{0.8, 	0.0, 	0.8,	1},		-- medium magenta	
	-- other pallets:
	[FillType.TREESAPLINGS]={0, 	0.39, 	0.09,	1},		-- Green
	[FillType.SUGARCANE] =	{0.014, 0.9, 	0.22,	1},		-- light Green
	[FillType.WOOL] =		{0.69, 	0.66, 	0.7, 	1},		-- light grey
	[FillType.EGG] =		{0.63, 	0.38, 	0.27,	1},		-- light brown
	[FillType.MILK] =		{0.59, 	0.62, 	0.62,	1},		-- light grey
	[FillType.POTATO] =		{0.15, 	0.05, 	0.05,	1},		-- light brown
	[FillType.SUGARBEET] =	{0.5, 	0.3, 	0.19,	1},		-- middle brown
	[FillType.POPLAR] =		{0.05, 	0.1, 	0.0,	1},		-- dark green
		}
	BaleSee.baleActivate   = true		-- 'True' means that the mod will show hot spots for bales.
	BaleSee.palletActivate = false		-- 'True' means that the mod will show hot spots for pallets.
	BaleSee.colorActivate  = true		-- 'True' means that the mod will create small colored circle hotspots.
										-- 'False' means that the mod will create hotspots that are small images of the objects.
	BaleSee.colPalActivate = true		-- Separate switch for the pallets: in this version always == colorActivate
	BaleSee.showHelp 	   = true 		-- show mods key help in F1-menu
	BaleSee.display 	   = 0			-- 0,1,2,3: display off,bales,pallets,both
	BaleSee.events = {} 				--
	BaleSee.legend = {} 				-- save hotspots for map legend
										-- width, height for hotspot icons:
	BaleSee.iconWid,
	BaleSee.iconHei 		= getNormalizedScreenValues(6,6)
										-- width, height for hotspot circles:
	BaleSee.spotWid,
	BaleSee.spotHei 		= getNormalizedScreenValues(5,5)
	BaleSee.bgImage 		= Utils.getFilename(BaleSee.icons.bgImage, BaleSee.modDir)
	BaleSee.uvB				= getNormalizedUVs({776, 776, 240, 240}) -- 8 + 3* 256 = 776
	BaleSee.uvP				= getNormalizedUVs({520, 520, 240, 240}) -- 8 + 2* 256 = 520
	BaleSee.ft 				= g_fillTypeManager.fillTypes

	--load settings from savegame folder
	local savegameIndex = g_currentMission.missionInfo.savegameIndex;
	local savegameFolderPath = g_currentMission.missionInfo.savegameDirectory;
	if savegameFolderPath == nil then
		savegameFolderPath = ('%ssavegame%d'):format(getUserProfileAppPath(), savegameIndex);
	end;
	if fileExists(savegameFolderPath .. '/careerSavegame.xml') then
		local key = "BaleSee";
		if fileExists(savegameFolderPath .. '/BaleSee.xml') then
			local xmlFile = loadXMLFile("BaleSee", savegameFolderPath .. "/BaleSee.xml", key);
			--BaleSee.baleActivate =  	Utils.getNoNil(getXMLBool(xmlFile, key.."#baleActivate"), true);
			--BaleSee.palletActivate =	Utils.getNoNil(getXMLBool(xmlFile, key.."#palletActivate"), false);
			BaleSee.display = 		Utils.getNoNil(getXMLInt(xmlFile, key.."#display"), 0);			
			BaleSee.colorActivate =	Utils.getNoNil(getXMLBool(xmlFile, key.."#colorActivate"), true);			
			BaleSee.showHelp = 	   	Utils.getNoNil(getXMLBool(xmlFile, key.."#showHelp"), true);			
			BaleSee.debug = 	   	Utils.getNoNil(getXMLBool(xmlFile, key.."#debug"), false);			
			delete(xmlFile);
		end;
		-- set switches from BaleSee.display value:
		BaleSee.baleActivate =   BaleSee.display %2 == 1		-- display == 1 or 3
		BaleSee.palletActivate = BaleSee.display > 1			-- display == 2 or 3
		BaleSee.colPalActivate = BaleSee.colorActivate
	end;
	if BaleSee.debug then
		BaleSee:makeLegend()
		print(string.format("** Bales: %s %s. Pallets: %s %s. F1-menu: %s",
			BaleSee.visible[BaleSee.baleActivate],   BaleSee.isIcon[BaleSee.colorActivate],
			BaleSee.visible[BaleSee.palletActivate], BaleSee.isIcon[BaleSee.colPalActivate],
			BaleSee.showHelp))
	end
end;
---------- "Mod main" ---------------------
addModEventListener(BaleSee);