--=======================================================================================================
--  BALESEE HOTSPOT FUNCTIONS
--
-- Purpose:		Allows bales and pallets to show up on the PDA map as hotspots.
-- Author:		Mmtrx		
-- Changelog:
--  v1.0		03.01.2019	original FS17 version by akuenzi (akuenzi@gmail.com)
--	v1.1.0		28.08.2019	updates for FS19, added user interface  
--  v1.1.1		17.09.2019  added pallette support.
--  v1.1.2		08.10.2019	save statistics / add legend in debug mode 
--=======================================================================================================
-- ---------------Hotspots for bales.-----------------------------------------------------------
function BaleSee:delete()
	BaleSee:delhot (self.mapHotspot)
	if BaleSee.debug then
		print("** in BaleSee:delete() "..tostring(self.id))
		--printCallstack()
		--DebugUtil.printTableRecursively(self,".",0,2)
		-- g_currentMission.itemsToSave contains all active bales 
	end
end;
function BaleSee:setNodeId(nodeId)
	--[[Add map hot spot for each bale so they can be found.  We put this here in the code, as it's
		a location that will end up getting called by load(), loadFromAttributesAndNodes(), and readStream().]]
	local isRoundbale = Utils.getNoNil(getUserAttribute(nodeId, "isRoundbale"), false)
	local color = 	BaleSee:getColor(self)
	local uv 	= 	BaleSee.uvB
	local image = 	BaleSee:getIcon(self)

	-- following hotspot attributes need to be set, depending on colorActivate:
	-- colorActivate	bgImage, 	image, 	color, 	uv, 	scal
	-- ----------------------------------------------------------
	-- 			true 	nil 		nil 	set 	set 	0.7
	--		   false 	set 		set 	nil 	nil 	1.0
	local nam = 	BaleSee.ft[self.fillType].title
	if g_gui.languageSuffix == "_de" then nam = nam.."-"
	else nam = nam.." "
	end
	nam = nam .. g_i18n:getText("unit_bale")
	-- Generate bale hotspot. Category = 6 to distinguish from others
	local hotspot = 	MapHotspot:new(nam, MapHotspot.CATEGORY_TOUR)

	if BaleSee.colorActivate then
		-- Hotspots will be small colored circles.
		hotspot:setIconScale(0.8)
		hotspot:setImage(nil, uv, color) 					-- colored circle 
		hotspot:setBackgroundImage(nil, uv, nil)			-- black circle
		hotspot:setSize(BaleSee.spotWid, BaleSee.spotHei)	-- size for small cicrcles
	else
		-- Hotspots will be small images/icons.
		hotspot:setImage(image, nil, nil)
		hotspot:setBackgroundImage(BaleSee.bgImage, nil, nil)	
		hotspot:setSize(BaleSee.iconWid, BaleSee.iconHei)	-- size for icons 
	end;
	hotspot.enabled = BaleSee.baleActivate						
	hotspot.verticalAlignment = Overlay.ALIGN_VERTICAL_MIDDLE		
	hotspot:setLinkedNode(nodeId)			-- also sets the x,z MapPos

	self.mapHotspot = hotspot 							-- property of the bale object
	g_currentMission.hud.ingameMap:addMapHotspot(hotspot) 
	table.insert(BaleSee.bHotspots, {hotspot,color,image,self.id}) 

	if BaleSee.debug then
		local x,y,z = getWorldTranslation(nodeId)
		print(string.format("** %s %s %s Bale %d/%d (%d) at %4.2f %4.2f.", 
			BaleSee.visible[BaleSee.baleActivate], BaleSee.isRound[isRoundbale],
			BaleSee.ft[self.fillType].name, self.id,
			nodeId, self.fillLevel, x, z))	
		--DebugUtil.printTableRecursively(self,".",0,2)
		--DebugUtil.printTableRecursively(hotspot,".",0,0)
	end;
end;

-- ----------------Manage Hotspots for pallets.--------------------------------------------------
function BaleSee:onDelete()		-- is called on delete for a pallet type object
	local typ = self.typeName
	if BaleSee.debug then
		local fillType = 	self:getFillUnitFillType(1)
		print(string.format("** Delete %s %s %d",
			BaleSee.ft[fillType].name, typ, self.rootNode))
		--local spec = self.specializations[11] 		--11: fillUnit
		--DebugUtil.printTableRecursively(spec,".",0,2)
	end
	-- remove hotspot from ingameMap and our own List
	local hot = self.mapHotspot
	if hot ~= nil then
		g_currentMission.hud.ingameMap:removeMapHotspot(hot);
		for i = 1, table.getn(BaleSee.pHotspots) do
			if BaleSee.pHotspots[i][1] == hot then
		    	table.remove(BaleSee.pHotspots, i)
		    	break
			end
		end
	end;
end;
function BaleSee:onLoadFinished(Savegame)
	--[[Add map hot spot for each pallet so they can be found.]]
	local nodeId = 	self.rootNode
	local fillType = 	self:getFillUnitFillType(1)
	local image = 		BaleSee:getImage(self)
	local color = 		BaleSee:getColor(self)
	local nam = 		BaleSee.ft[fillType].title
	-- Generate pallet hotspot. Category = 6 to distinguish from other
	local hotspot = 	MapHotspot:new(nam, MapHotspot.CATEGORY_TOUR)
	
	if BaleSee.colPalActivate then
		-- Hotspots will be small colored blocks.
		hotspot:setIconScale(0.7)
		hotspot:setImage 		  (nil, BaleSee.uvP, color)
		hotspot:setBackgroundImage(nil, BaleSee.uvP, nil)	
		hotspot:setSize(BaleSee.spotWid, BaleSee.spotHei)
	else
		-- Hotspots will be small images/icons.
		hotspot:setImage(image, nil, nil)
		hotspot:setBackgroundImage(BaleSee.bgImage, nil, nil)	
		hotspot:setSize(BaleSee.iconWid, BaleSee.iconHei)
	end;
	hotspot:setLinkedNode(nodeId)			-- also sets the x,z MapPos
	hotspot.verticalAlignment = Overlay.ALIGN_VERTICAL_MIDDLE		
	hotspot.enabled = BaleSee.palletActivate						
	g_currentMission.hud.ingameMap:addMapHotspot(hotspot) 
	
	self.mapHotspot = hotspot 				-- property of the pallet object
	table.insert(BaleSee.pHotspots, {hotspot,color,image}) 

	if BaleSee.debug then
		local x,y,z = getWorldTranslation(nodeId)
		print(string.format("** %s %s Pallet %d at %4.2f %4.2f", 
			BaleSee.visible[BaleSee.palletActivate], BaleSee.ft[fillType].name, nodeId, x, z))
		-- print(string.format( "***config/image: %s, %s", self.configFileName ,image))
		--	local spec = self.spec_fillUnit
		--	DebugUtil.printTableRecursively(spec,".",0,2)
	end;			
end;
function BaleSee:onChangedFillType(fillUnitIndex, fillTypeIndex, oldFillTypeIndex)
	-- to set fillType of newly filled fillablePallet (wool / potato / sugarbeet)
	-- only for existing hotspots
	local hotspot = self.mapHotspot
		-- adjust hotspot only if fillType changes from UNKNOWN to something
	if oldFillTypeIndex ~= FillType.UNKNOWN or hotspot == nil then
		return
	end
	local color, image = BaleSee:getColor(self), BaleSee:getImage(self)
	local fillType = 	self:getFillUnitFillType(1)
	if BaleSee.colPalActivate then
		hotspot:setImage(nil, BaleSee.uvP, color) 	-- change color.
	else			
		hotspot:setImage(image, nil, nil) 			-- change small image/icon.
	end;
	-- change also in our table of hotspots:
	for i,h in ipairs(BaleSee.pHotspots) do
		if h[1] == hotspot then
			-- BaleSee.pHotspots[i] = {h[1],color,image}
			h[2],h[3] = color,image 				-- should be enough
			hotspot:setText(BaleSee.ft[fillType].title, true, false)  --(name,hidden,alwaysshow)
			break
		end
	end
	if BaleSee.debug then
		print(string.format("** filltype change(%d,%d,%d) for %s pallet %d to %s", 
		fillUnitIndex, fillTypeIndex, oldFillTypeIndex,
		BaleSee.ft[oldFillTypeIndex].name,
	 	self.rootNode, BaleSee.ft[fillTypeIndex].name))
	end
end
