--=======================================================================================================
--  BALESEE USER INTERFACE FUNCTIONS
--
-- Purpose:		Allows bales and pallets to show up on the PDA map as hotspots.
-- Author:		Mmtrx		
-- Changelog:
--  v1.0		03.01.2019	original FS17 version by akuenzi (akuenzi@gmail.com)
--	v1.1.0		28.08.2019	updates for FS19, added user interface  
--  v1.1.1		17.09.2019  added pallette support.
--  v1.1.2		08.10.2019	save statistics / add legend in debug mode 
--=======================================================================================================
-------------------- User interface functions ---------------------------------------------------
function BaleSee:registerActionEventsPlayer()
	-- gets called when player leaves vehicle
	-- register action events
	local result, eventId 
	for e, f in pairs(BaleSee.ACTIONS) do
		result, eventId = InputBinding.registerActionEvent(g_inputBinding,e,self,f,false,true,false,true)
		if result then
			table.insert(BaleSee.events, eventId);
			g_inputBinding.events[eventId].displayIsVisible = BaleSee.showHelp;
    	end
    end
	if BaleSee.debug then
	--	print("--BaleSee Debug ... BaleSee:registerActionEventsPlayer(): BaleSee.events:");
	--	DebugUtil.printTableRecursively(BaleSee.events,"-",0,0)
	end;	
end;
function BaleSee:removeActionEventsPlayer()
	-- gets called when player enters vehicle
	BaleSee.events = {};
	if BaleSee.debug then
	--	print("--BaleSee Debug ... BaleSee:removeActionEventsPlayer()");
	end;
end;
function BaleSee:saveSettings()
	local savegameIndex = g_currentMission.missionInfo.savegameIndex;
	local savegameFolderPath = g_currentMission.missionInfo.savegameDirectory;
	if savegameFolderPath == nil then
		savegameFolderPath = ('%ssavegame%d'):format(getUserProfileAppPath(), savegameIndex);
	end;
	local key = "BaleSee";
	local xmlFile = createXMLFile("BaleSee", savegameFolderPath .. "/BaleSee.xml", key);
	setXMLInt(xmlFile, key .. "#display",  BaleSee.display);
	setXMLBool(xmlFile, key .. "#colorActivate", BaleSee.colorActivate);		
	setXMLBool(xmlFile, key .. "#showHelp",BaleSee.showHelp);
	saveXMLFile(xmlFile);
	delete(xmlFile);
	-- print bale/pallet statistics
	BaleSee:printStats()
	if BaleSee.debug then
		print("** BaleSee:saved settings to " ..savegameFolderPath .."/BaleSee.xml");
	end;
end;

function BaleSee:actionbs_Bale(actionName, keyStatus, arg3, arg4, arg5)
	-- switch BaleSee.display to next state 
	BaleSee.display = (BaleSee.display +1) %4
	local bA, pA = BaleSee.baleActivate, BaleSee.palletActivate
	BaleSee.baleActivate =   BaleSee.display %2 == 1		-- true, if display == 1 or 3
	BaleSee.palletActivate = BaleSee.display > 1			-- true, if display == 2 or 3
	if BaleSee.baleActivate   ~= bA then 
		BaleSee:toggleVis(BaleSee.bHotspots, BaleSee.baleActivate) 
		if BaleSee.debug then
			print("** set baleActivate to "..tostring(BaleSee.baleActivate));end;
	end
	if BaleSee.palletActivate ~= pA then 
		BaleSee:toggleVis(BaleSee.pHotspots, BaleSee.palletActivate) 
		if BaleSee.debug then
			print("** set palletActivate to "..tostring(BaleSee.palletActivate));end;
	end
end;
function BaleSee:actionbs_Color(actionName, keyStatus, arg3, arg4, arg5)
	-- toggle BaleSee.colorActivate
	BaleSee.colorActivate = not BaleSee.colorActivate
	if BaleSee.debug then
		print("** set colorActivate to " .. tostring(BaleSee.colorActivate));
	end
	if BaleSee.colorActivate then
	-- for each bale hotspot: reset it to circle marker
		for _, h in pairs(BaleSee.bHotspots) do
			-- h is {hotspot, color, image}
			hs = h[1]
			--remove image, bgImage, set uv and color
			hs:setImage 		 (nil, BaleSee.uvB, h[2])
			hs:setBackgroundImage(nil, BaleSee.uvB, nil)	
			hs:setIconScale		 (0.7)
			hs:setSize			 (BaleSee.spotWid, BaleSee.spotHei)	
		end
	else    -- reset hotspots to Icons:
		for _, h in pairs(BaleSee.bHotspots) do
			hs = h[1]
			--remove uv, col. Set image, bgImage 
			hs:setImage 		 (h[3], nil, nil)
			hs:setBackgroundImage(BaleSee.bgImage, nil, nil)	
			hs:setIconScale 	 (1)
			hs:setSize			 (BaleSee.iconWid, BaleSee.iconHei)	
		end
	end
	-- same for pallets:
	BaleSee:togglePalCol()
end;
function BaleSee:actionbs_Help(actionName, keyStatus, arg3, arg4, arg5)
	-- toggle BaleSee.showHelp
	BaleSee.showHelp = not BaleSee.showHelp
	if BaleSee.events then
		for _,e in pairs(BaleSee.events) do
			g_inputBinding:setActionEventTextVisibility(e, BaleSee.showHelp)
		end
    end
	if BaleSee.debug then
		-- toggle legend hotspots on/off
		BaleSee:toggleVis(BaleSee.legend, BaleSee.showHelp) 
		print("** set showHelp to " .. tostring(BaleSee.showHelp)
			..". inputHelp.numUsedEntries = "..tostring(g_currentMission.hud.inputHelp.numUsedEntries));
	end
end;
